@@ -13,7 +13,6 @@ pub enum Error {
         other_user: Option<SudoString>,
     },
     SelfCheck,
-    KernelCheck,
     CommandNotFound(PathBuf),
     InvalidCommand(PathBuf),
     ChDirNotAllowed {
@@ -32,6 +31,8 @@ pub enum Error {
     MaxAuthAttempts(usize),
     PathValidation(PathBuf),
     StringValidation(String),
+    #[cfg(feature = "apparmor")]
+    AppArmor(String, std::io::Error),
 }
 
 impl fmt::Display for Error {
@@ -59,7 +60,6 @@ impl fmt::Display for Error {
             Error::SelfCheck => {
                 f.write_str("sudo must be owned by uid 0 and have the setuid bit set")
             }
-            Error::KernelCheck => f.write_str("sudo-rs needs a Linux kernel newer than v5.9"),
             Error::CommandNotFound(p) => write!(f, "'{}': command not found", p.display()),
             Error::InvalidCommand(p) => write!(f, "'{}': invalid command", p.display()),
             Error::UserNotFound(u) => write!(f, "user '{u}' not found"),
@@ -103,6 +103,10 @@ impl fmt::Display for Error {
             Error::PathValidation(path) => {
                 write!(f, "invalid path: {path:?}")
             }
+            #[cfg(feature = "apparmor")]
+            Error::AppArmor(profile, e) => {
+                write!(f, "unable to change AppArmor profile to {profile}: {e}")
+            }
         }
     }
 }
