@@ -2,20 +2,18 @@
 //! want any copies to leak (that we would then need to zeroize).
 use std::{
     alloc::{self, Layout},
-    mem,
     ptr::NonNull,
     slice,
 };
 
 const SIZE: usize = super::sys::PAM_MAX_RESP_SIZE as usize;
-const ALIGN: usize = mem::align_of::<u8>();
 
 pub struct PamBuffer(NonNull<[u8; SIZE]>);
 
-fn layout() -> Layout {
-    // does not panic with the given arguments; also see unit test at the bottom
-    Layout::from_size_align(SIZE, ALIGN).unwrap()
-}
+const LAYOUT: Layout = match Layout::from_size_align(SIZE, 1) {
+    Ok(layout) => layout,
+    Err(_) => unreachable!(),
+};
 
 impl PamBuffer {
     // consume this buffer and return its internal pointer
@@ -48,7 +46,7 @@ impl Default for PamBuffer {
         if let Some(nn) = NonNull::new(res) {
             PamBuffer(nn.cast())
         } else {
-            alloc::handle_alloc_error(layout())
+            alloc::handle_alloc_error(LAYOUT)
         }
     }
 }
@@ -96,6 +94,7 @@ fn wipe_memory(memory: &mut [u8]) {
     atomic::compiler_fence(atomic::Ordering::SeqCst);
 }
 
+#[allow(clippy::undocumented_unsafe_blocks)]
 #[cfg(test)]
 mod test {
     use super::PamBuffer;
@@ -123,9 +122,4 @@ mod test {
         assert!(fix[3..].iter().all(|&x| x == 0));
         std::mem::drop(fix);
     }
-
-    #[test]
-    fn layout_does_not_panic() {
-        let _ = super::layout();
-    }
 }
