@@ -1,5 +1,6 @@
 use crate::common::resolve::CurrentUser;
 use crate::common::{CommandAndArguments, Context};
+use crate::exec::Umask;
 use crate::sudo::{
     cli::{SudoAction, SudoRunOptions},
     env::environment::{get_target_environment, Environment},
@@ -132,8 +133,10 @@ fn create_test_context(sudo_options: SudoRunOptions) -> Context {
         process: Process::new(),
         use_session_records: false,
         use_pty: true,
-        password_feedback: false,
+        noexec: false,
         bell: false,
+        umask: Umask::Preserve,
+        files_to_edit: vec![],
     }
 }
 
@@ -170,6 +173,10 @@ fn test_environment_variable_filtering() {
                 use_pty: true,
                 chdir: crate::sudoers::DirChange::Strict(None),
                 trust_environment: false,
+                umask: crate::exec::Umask::Preserve,
+                #[cfg(feature = "apparmor")]
+                apparmor_profile: None,
+                noexec: false,
             },
         )
         .unwrap();
