@@ -1,5 +1,6 @@
 use super::ast_names::UserFriendly;
 use super::basic_parser::*;
+use super::char_stream::advance;
 use super::tokens::*;
 use crate::common::SudoString;
 use crate::common::{
@@ -96,6 +97,18 @@ pub enum EnvironmentControl {
     Nosetenv = HARDENED_ENUM_VALUE_2,
 }
 
+#[derive(Copy, Clone, Default, PartialEq)]
+#[cfg_attr(test, derive(Debug, Eq))]
+#[repr(u32)]
+pub enum ExecControl {
+    #[default]
+    Implicit = HARDENED_ENUM_VALUE_0,
+    // PASSWD:
+    Exec = HARDENED_ENUM_VALUE_1,
+    // NOPASSWD:
+    Noexec = HARDENED_ENUM_VALUE_2,
+}
+
 /// Commands in /etc/sudoers can have attributes attached to them, such as NOPASSWD, NOEXEC, ...
 #[derive(Default, Clone, PartialEq)]
 #[cfg_attr(test, derive(Debug, Eq))]
@@ -103,16 +116,14 @@ pub struct Tag {
     pub(super) authenticate: Authenticate,
     pub(super) cwd: Option<ChDir>,
     pub(super) env: EnvironmentControl,
+    pub(super) apparmor_profile: Option<String>,
+    pub(super) noexec: ExecControl,
 }
 
 impl Tag {
     pub fn needs_passwd(&self) -> bool {
         matches!(self.authenticate, Authenticate::None | Authenticate::Passwd)
     }
-
-    pub fn allows_setenv(&self) -> bool {
-        matches!(self.env, EnvironmentControl::Setenv)
-    }
 }
 
 /// Commands with attached attributes.
@@ -311,16 +322,13 @@ impl Parse for UserSpecifier {
 
         // if we see a quote, first parse the quoted text as a token and then
         // re-parse whatever we found inside; this is a lazy solution but it works
-        let begin_pos = stream.get_pos();
         if stream.eat_char('"') {
+            let begin_pos = stream.get_pos();
             let Unquoted(text, _): Unquoted<Username> = expect_nonterminal(stream)?;
-            let result = parse_user(&mut CharStream::new(text.chars()));
-            if result.is_err() {
-                unrecoverable!(pos = begin_pos, stream, "invalid user")
-            }
+            let result = parse_user(&mut CharStream::new_with_pos(&text, begin_pos))?;
             expect_syntax('"', stream)?;
 
-            result
+            Ok(result)
         } else {
             parse_user(stream)
         }
@@ -380,14 +388,15 @@ impl Parse for MetaOrTag {
         };
 
         let result: Modifier = match keyword.as_str() {
-            // we do not support these, and that should make sudo-rs "fail safe"
-            "INTERCEPT" | "NOEXEC" => unrecoverable!(
+            // we do not support this, and that should make sudo-rs "fail safe"
+            "INTERCEPT" => unrecoverable!(
                 pos = start_pos,
                 stream,
-                "NOEXEC and INTERCEPT are not supported by sudo-rs"
+                "INTERCEPT is not supported by sudo-rs"
             ),
             // this is less fatal
-            "LOG_INPUT" | "NOLOG_INPUT" | "LOG_OUTPUT" | "NOLOG_OUTPUT" | "MAIL" | "NOMAIL" => {
+            "LOG_INPUT" | "NOLOG_INPUT" | "LOG_OUTPUT" | "NOLOG_OUTPUT" | "MAIL" | "NOMAIL"
+            | "FOLLOW" => {
                 eprintln_ignore_io_error!(
                     "warning: {} tags are ignored by sudo-rs",
                     keyword.as_str()
@@ -395,9 +404,11 @@ impl Parse for MetaOrTag {
                 switch(|_| {})?
             }
 
-            // 'FOLLOW' and 'NOFOLLOW' are only usable in a sudoedit context, which will result in
-            // a parse error elsewhere. 'EXEC' and 'NOINTERCEPT' are the default behaviour.
-            "FOLLOW" | "NOFOLLOW" | "EXEC" | "NOINTERCEPT" => switch(|_| {})?,
+            // 'NOFOLLOW' and 'NOINTERCEPT' are the default behaviour.
+            "NOFOLLOW" | "NOINTERCEPT" => switch(|_| {})?,
+
+            "EXEC" => switch(|tag| tag.noexec = ExecControl::Exec)?,
+            "NOEXEC" => switch(|tag| tag.noexec = ExecControl::Noexec)?,
 
             "SETENV" => switch(|tag| tag.env = EnvironmentControl::Setenv)?,
             "NOSETENV" => switch(|tag| tag.env = EnvironmentControl::Nosetenv)?,
@@ -418,11 +429,23 @@ impl Parse for MetaOrTag {
             "ROLE" | "TYPE" => unrecoverable!(
                 pos = start_pos,
                 stream,
-                "role based access control is not yet supported by sudo-rs"
+                "SELinux role based access control is not yet supported by sudo-rs"
             ),
 
+            "APPARMOR_PROFILE" => {
+                expect_syntax('=', stream)?;
+                let StringParameter(profile) = expect_nonterminal(stream)?;
+                Box::new(move |tag| tag.apparmor_profile = Some(profile.clone()))
+            }
+
             "ALL" => return make(MetaOrTag(All)),
-            alias => return make(MetaOrTag(Alias(alias.to_string()))),
+            alias => {
+                if is_syntax('=', stream)? {
+                    unrecoverable!(pos = start_pos, stream, "unsupported modifier '{}'", alias);
+                } else {
+                    return make(MetaOrTag(Alias(alias.to_string())));
+                }
+            }
         };
 
         make(MetaOrTag(Only(result)))
@@ -435,9 +458,9 @@ impl Parse for MetaOrTag {
 /// ```
 impl Parse for CommandSpec {
     fn parse(stream: &mut CharStream) -> Parsed<Self> {
+        use Qualified::Allow;
         let mut tags = vec![];
         while let Some(MetaOrTag(keyword)) = try_nonterminal(stream)? {
-            use Qualified::Allow;
             match keyword {
                 Meta::Only(modifier) => tags.push(modifier),
                 Meta::All => return make(CommandSpec(tags, Allow(Meta::All))),
@@ -448,28 +471,6 @@ impl Parse for CommandSpec {
             }
         }
 
-        let start_pos = stream.get_pos();
-        if let Some(Username(keyword)) = try_nonterminal(stream)? {
-            if keyword == "sudoedit" {
-                // note: special behaviour of forward slashes in wildcards, tread carefully
-                unrecoverable!(pos = start_pos, stream, "sudoedit is not yet supported");
-            } else if keyword == "list" {
-                unrecoverable!(pos = start_pos, stream, "list is not yet supported");
-            } else if keyword.starts_with("sha") {
-                unrecoverable!(
-                    pos = start_pos,
-                    stream,
-                    "digest specifications are not supported"
-                )
-            } else {
-                unrecoverable!(
-                    pos = start_pos,
-                    stream,
-                    "expected command but found {keyword}"
-                )
-            };
-        }
-
         let cmd: Spec<Command> = expect_nonterminal(stream)?;
 
         make(CommandSpec(tags, cmd))
@@ -492,7 +493,7 @@ impl Parse for (SpecList<Hostname>, Vec<(Option<RunAs>, CommandSpec)>) {
 }
 
 /// A hostname, runas specifier, commandspec combination can occur multiple times in a single
-/// sudoer line (seperated by ":")
+/// sudoer line (separated by ":")
 impl Many for (SpecList<Hostname>, Vec<(Option<RunAs>, CommandSpec)>) {
     const SEP: char = ':';
 }
@@ -516,7 +517,7 @@ impl Parse for (Option<RunAs>, CommandSpec) {
 }
 
 /// A runas specifier, commandspec combination can occur multiple times in a single
-/// sudoer line (seperated by ","); there is some ambiguity in the original grammar:
+/// sudoer line (separated by ","); there is some ambiguity in the original grammar:
 /// commands can also occur multiple times; we parse that here as if they have an omitted
 /// "runas" specifier (which has to be placed correctly during the AST analysis phase)
 impl Many for (Option<RunAs>, CommandSpec) {}
@@ -565,7 +566,13 @@ impl Parse for Sudo {
         }
 
         let start_pos = stream.get_pos();
-        if let Some(users) = maybe(try_nonterminal::<SpecList<_>>(stream))? {
+        if stream.peek() == Some('"') {
+            // a quoted userlist follows; this forces us to read a userlist
+            let users = expect_nonterminal(stream)?;
+            let permissions = expect_nonterminal(stream)?;
+            make(Sudo::Spec(PermissionSpec { users, permissions }))
+        } else if let Some(users) = maybe(try_nonterminal::<SpecList<_>>(stream))? {
+            // this could be the start of a Defaults or Alias definition, so distinguish.
             // element 1 always exists (parse_list fails on an empty list)
             let key = &users[0];
             if let Some(directive) = maybe(get_directive(key, stream, start_pos))? {
@@ -630,7 +637,7 @@ fn parse_include(stream: &mut CharStream) -> Parsed<Sudo> {
 
 /// grammar:
 /// ```text
-/// name = definition [ : name = definiton [ : ... ] ]
+/// name = definition [ : name = definition [ : ... ] ]
 /// ```
 ///
 impl<T> Parse for Def<T>
@@ -681,15 +688,26 @@ fn get_directive(
         "Host_Alias" => make(HostAlias(expect_nonterminal(stream)?)),
         "Cmnd_Alias" | "Cmd_Alias" => make(CmndAlias(expect_nonterminal(stream)?)),
         "Runas_Alias" => make(RunasAlias(expect_nonterminal(stream)?)),
-        "Defaults" => {
-            //HACK: this avoids having to add "Defaults@" etc as separate tokens; but relying
-            //on positional information during parsing is of course, cheating.
+        _ if keyword.starts_with("Defaults") => {
+            //HACK #1: no space is allowed between "Defaults" and '!>@:'. The below avoids having to
+            //add "Defaults!" etc as separate tokens; but relying on positional information during
+            //parsing is of course, cheating.
+            //HACK #2: '@' can be part of a username, so it will already have been parsed;
+            //an acceptable hostname is subset of an acceptable username, so that's actually OK.
+            //This resolves an ambiguity in the grammar similarly to how MetaOrTag does that.
+            const DEFAULTS_LEN: usize = "Defaults".len();
             let allow_scope_modifier = stream.get_pos().0 == begin_pos.0
-                && stream.get_pos().1 - begin_pos.1 == "Defaults".len();
+                && (stream.get_pos().1 - begin_pos.1 == DEFAULTS_LEN
+                    || keyword.len() > DEFAULTS_LEN);
 
             let scope = if allow_scope_modifier {
-                if is_syntax('@', stream)? {
-                    ConfigScope::Host(expect_nonterminal(stream)?)
+                if keyword[DEFAULTS_LEN..].starts_with('@') {
+                    let inner_stream = &mut CharStream::new_with_pos(
+                        &keyword[DEFAULTS_LEN + 1..],
+                        advance(begin_pos, DEFAULTS_LEN + 1),
+                    );
+
+                    ConfigScope::Host(expect_nonterminal(inner_stream)?)
                 } else if is_syntax(':', stream)? {
                     ConfigScope::User(expect_nonterminal(stream)?)
                 } else if is_syntax('!', stream)? {
@@ -776,13 +794,13 @@ impl Parse for defaults::SettingsModifier {
             let DefaultName(name) = expect_nonterminal(stream)?;
             let Some(modifier) = defaults::negate(&name) else {
                 if defaults::set(&name).is_some() {
-                    unrecoverable!(pos = value_pos, stream, "unknown setting: '{name}'");
-                } else {
                     unrecoverable!(
                         pos = value_pos,
                         stream,
                         "'{name}' cannot be used in a boolean context"
                     );
+                } else {
+                    unrecoverable!(pos = value_pos, stream, "unknown setting: '{name}'");
                 }
             };
 
