@@ -13,11 +13,11 @@ use std::collections::{HashMap, HashSet};
 use std::io;
 use std::path::{Path, PathBuf};
 
-use crate::common::resolve::resolve_path;
+use crate::common::resolve::{is_valid_executable, resolve_path};
 use crate::defaults;
 use crate::log::auth_warn;
 use crate::system::interface::{GroupId, UnixGroup, UnixUser, UserId};
-use crate::system::{self, can_execute};
+use crate::system::{self, audit};
 use ast::*;
 use tokens::*;
 
@@ -59,6 +59,7 @@ pub struct Request<'a, User: UnixUser, Group: UnixGroup> {
 }
 
 pub struct ListRequest<'a, User: UnixUser, Group: UnixGroup> {
+    pub inspected_user: &'a User,
     pub target_user: &'a User,
     pub target_group: &'a Group,
 }
@@ -97,23 +98,34 @@ impl Sudoers {
         &mut self,
         hostname: &system::Hostname,
         requesting_user: &User,
-        target_user: &User,
+        target_user: Option<&User>,
     ) {
         let customisers = std::mem::take(&mut self.customisers.non_cmnd);
 
         let host_matcher = &match_token(hostname);
-        let user_matcher = &match_user(requesting_user);
-        let runas_matcher = &match_user(target_user);
-
         let host_aliases = get_aliases(&self.aliases.host, host_matcher);
+
+        let user_matcher = &match_user(requesting_user);
         let user_aliases = get_aliases(&self.aliases.user, user_matcher);
-        let runas_aliases = get_aliases(&self.aliases.runas, runas_matcher);
+
+        let runas_matcher_aliases = target_user.map(|target_user| {
+            let runas_matcher = match_user(target_user);
+            let runas_aliases = get_aliases(&self.aliases.runas, &runas_matcher);
+
+            (runas_matcher, runas_aliases)
+        });
 
         let match_scope = |scope| match scope {
             ConfigScope::Generic => true,
             ConfigScope::Host(list) => find_item(&list, host_matcher, &host_aliases).is_some(),
             ConfigScope::User(list) => find_item(&list, user_matcher, &user_aliases).is_some(),
-            ConfigScope::RunAs(list) => find_item(&list, runas_matcher, &runas_aliases).is_some(),
+            ConfigScope::RunAs(list) => {
+                runas_matcher_aliases
+                    .as_ref()
+                    .is_some_and(|(runas_matcher, runas_aliases)| {
+                        find_item(&list, runas_matcher, runas_aliases).is_some()
+                    })
+            }
             ConfigScope::Command(_list) => {
                 unreachable!("command-specific defaults are filtered out")
             }
@@ -149,7 +161,7 @@ impl Sudoers {
         on_host: &system::Hostname,
         request: Request<User, Group>,
     ) -> Judgement {
-        self.specify_host_user_runas(on_host, am_user, request.user);
+        self.specify_host_user_runas(on_host, am_user, Some(request.user));
         self.specify_command(request.command, request.arguments);
 
         // exception: if user is root or does not switch users, NOPASSWD is implied
@@ -170,21 +182,37 @@ impl Sudoers {
     }
 
     pub fn check_list_permission<User: UnixUser + PartialEq<User>, Group: UnixGroup>(
-        &self,
+        &mut self,
         invoking_user: &User,
         hostname: &system::Hostname,
         request: ListRequest<User, Group>,
     ) -> Authorization {
-        // exception: if user is root or does not switch users, NOPASSWD is implied
-        let skip_passwd = invoking_user.is_root()
-            || (request.target_user == invoking_user
-                && in_group(invoking_user, request.target_group));
+        let skip_passwd;
+        let mut flags = if request.inspected_user != invoking_user {
+            skip_passwd = invoking_user.is_root();
 
-        let mut flags = self
-            .matching_user_specs(invoking_user, hostname)
-            .flatten()
-            .map(|(_, (tag, _))| tag)
-            .max_by_key(|tag| !tag.needs_passwd());
+            self.check(
+                invoking_user,
+                hostname,
+                Request {
+                    user: request.inspected_user,
+                    group: &request.inspected_user.group(),
+                    command: Path::new("list"),
+                    arguments: &[],
+                },
+            )
+            .flags
+            .or(invoking_user.is_root().then(Tag::default))
+        } else {
+            skip_passwd = invoking_user.is_root()
+                || (request.target_user == invoking_user
+                    && in_group(invoking_user, request.target_group));
+
+            self.matching_user_specs(invoking_user, hostname)
+                .flatten()
+                .map(|(_, (tag, _))| tag)
+                .max_by_key(|tag| !tag.needs_passwd())
+        };
 
         if let Some(tag) = flags.as_mut() {
             if skip_passwd {
@@ -198,10 +226,12 @@ impl Sudoers {
     }
 
     pub fn check_validate_permission<User: UnixUser + PartialEq<User>>(
-        &self,
+        &mut self,
         invoking_user: &User,
         hostname: &system::Hostname,
     ) -> Authorization {
+        self.specify_host_user_runas(hostname, invoking_user, None);
+
         // exception: if user is root, NOPASSWD is implied
         let skip_passwd = invoking_user.is_root();
 
@@ -260,33 +290,62 @@ impl Sudoers {
         user_specs.flat_map(|cmd_specs| group_cmd_specs_per_runas(cmd_specs, &self.aliases.cmnd))
     }
 
-    pub(crate) fn solve_editor_path<User: UnixUser + PartialEq<User>>(
+    pub(crate) fn visudo_editor_path<User: UnixUser + PartialEq<User>>(
         mut self,
         on_host: &system::Hostname,
         am_user: &User,
         target_user: &User,
-    ) -> Option<PathBuf> {
-        self.specify_host_user_runas(on_host, am_user, target_user);
-        if self.settings.env_editor() {
-            for key in ["SUDO_EDITOR", "VISUAL", "EDITOR"] {
-                if let Some(var) = std::env::var_os(key) {
-                    let path = Path::new(&var);
-                    if can_execute(path) {
-                        return Some(path.to_owned());
-                    }
-                    let path = resolve_path(
-                        path,
-                        &std::env::var("PATH").unwrap_or(env!("SUDO_PATH_DEFAULT").to_string()),
-                    );
-                    if let Some(path) = path {
-                        return Some(path);
-                    }
-                }
+    ) -> PathBuf {
+        self.specify_host_user_runas(on_host, am_user, Some(target_user));
+
+        select_editor(&self.settings, self.settings.env_editor())
+    }
+}
+
+/// Retrieve the chosen editor from a settings object, filtering based on whether the
+/// environment is trusted (sudoedit) or maybe less so (visudo)
+fn select_editor(settings: &Settings, trusted_env: bool) -> PathBuf {
+    let blessed_editors = settings.editor();
+
+    let is_whitelisted = |path: &Path| -> bool {
+        trusted_env || blessed_editors.split(':').any(|x| Path::new(x) == path)
+    };
+
+    // find editor in environment, if possible
+
+    for key in ["SUDO_EDITOR", "VISUAL", "EDITOR"] {
+        if let Some(editor) = std::env::var_os(key) {
+            let editor = PathBuf::from(editor);
+
+            let editor = if is_valid_executable(&editor) {
+                editor
+            } else if let Some(editor) = resolve_path(
+                &editor,
+                &std::env::var("PATH").unwrap_or(env!("SUDO_PATH_DEFAULT").to_string()),
+            ) {
+                editor
+            } else {
+                continue;
+            };
+
+            if is_whitelisted(&editor) {
+                return editor;
             }
         }
-
-        None
     }
+
+    // no acceptable editor found in environment, fallback on config
+
+    for editor in blessed_editors.split(':') {
+        let editor = PathBuf::from(editor);
+        if is_valid_executable(&editor) {
+            return editor;
+        }
+    }
+
+    // fallback on hardcoded path -- always provide something to the caller
+
+    PathBuf::from(defaults::SYSTEM_EDITOR)
 }
 
 // a `take_while` variant that does not consume the first non-matching item
@@ -336,16 +395,16 @@ fn read_sudoers<R: io::Read>(mut reader: R) -> io::Result<Vec<basic_parser::Pars
 
     use basic_parser::parse_lines;
     use char_stream::*;
-    Ok(parse_lines(&mut CharStream::new(buffer.chars())))
+    Ok(parse_lines(&mut CharStream::new(&buffer)))
 }
 
 fn open_sudoers(path: &Path) -> io::Result<Vec<basic_parser::Parsed<Sudo>>> {
-    let source = crate::system::secure_open(path, false)?;
+    let source = audit::secure_open_sudoers(path, false)?;
     read_sudoers(source)
 }
 
 fn open_subsudoers(path: &Path) -> io::Result<Vec<basic_parser::Parsed<Sudo>>> {
-    let source = crate::system::secure_open(path, true)?;
+    let source = audit::secure_open_sudoers(path, true)?;
     read_sudoers(source)
 }
 
@@ -557,11 +616,11 @@ fn match_group_alias(group: &impl UnixGroup) -> impl Fn(&UserSpecifier) -> bool
 
 fn match_token<T: basic_parser::Token + std::ops::Deref<Target = String>>(
     text: &str,
-) -> (impl Fn(&T) -> bool + '_) {
+) -> impl Fn(&T) -> bool + '_ {
     move |token| token.as_str() == text
 }
 
-fn match_command<'a>((cmd, args): (&'a Path, &'a [String])) -> (impl Fn(&Command) -> bool + 'a) {
+fn match_command<'a>((cmd, args): (&'a Path, &'a [String])) -> impl Fn(&Command) -> bool + 'a {
     let opts = glob::MatchOptions {
         require_literal_separator: true,
         ..glob::MatchOptions::new()
