@@ -36,6 +36,10 @@ impl UnixUser for Named {
     fn is_root(&self) -> bool {
         self.0 == "root"
     }
+    type Group = Named;
+    fn group(&self) -> Named {
+        Self(self.0)
+    }
 }
 
 impl UnixGroup for Named {
@@ -58,7 +62,7 @@ macro_rules! request {
 
 macro_rules! sudoer {
     ($($e:expr),*) => {
-        parse_lines(&mut CharStream::new([$($e),*, ""].join("\n").chars()))
+        parse_lines(&mut CharStream::new(&[$($e),*, ""].join("\n")))
             .into_iter()
             .map(|x| Ok::<_,basic_parser::Status>(x.unwrap()))
     }
@@ -72,7 +76,7 @@ fn parse_line(s: &str) -> Sudo {
 
 /// Returns `None` if a syntax error is encountered
 fn try_parse_line(s: &str) -> Option<Sudo> {
-    parse_lines(&mut CharStream::new([s, ""].join("").chars()))
+    parse_lines(&mut CharStream::new(&[s, ""].join("")))
         .into_iter()
         .next()?
         .ok()
@@ -242,7 +246,7 @@ fn permission_test() {
 
     pass!(["Runas_Alias \\"," TIME=%wheel \\",",sudo # hallo","user ALL \\","=(TIME) ALL"], "user" => request! { wheel, wheel }, "vm"; "/bin/ls");
 
-    // test the less-intuitive "substition-like" alias mechanism
+    // test the less-intuitive "substitution-like" alias mechanism
     FAIL!(["User_Alias FOO=!user", "ALL, FOO ALL=ALL"], "user" => root(), "vm"; "/bin/ls");
     pass!(["User_Alias FOO=!user", "!FOO ALL=ALL"], "user" => root(), "vm"; "/bin/ls");
 
@@ -250,6 +254,18 @@ fn permission_test() {
     pass!(["a\\,b ALL=ALL"], "a,b" => request! { root, root }, "server"; "/bin/foo");
     pass!(["\"a,b\" ALL=ALL"], "a,b" => request! { root, root }, "server"; "/bin/foo");
     pass!(["\"a\\b\" ALL=ALL"], "a\\b" => request! { root, root }, "server"; "/bin/foo");
+
+    // special chacters
+    pass!(["foo@machine.name ALL=ALL"], "foo@machine.name" => request! { root, root }, "server"; "/bin/foo");
+    pass!(["fnord$ ALL=ALL"], "fnord$" => request! { root, root }, "server"; "/bin/foo");
+
+    // apparmor
+    #[cfg(feature = "apparmor")]
+    pass!(["ALL ALL=(ALL:ALL) APPARMOR_PROFILE=unconfined ALL"], "user" => root(), "server"; "/bin/bar" => [apparmor_profile: Some("unconfined".to_string())]);
+
+    // list
+    pass!(["ALL ALL=(ALL:ALL) /bin/ls, list"], "user" => root(), "server"; "list");
+    FAIL!(["ALL ALL=(ALL:ALL) ALL, !list"], "user" => root(), "server"; "list");
 }
 
 #[test]
@@ -268,7 +284,7 @@ fn default_bool_test() {
     sudoers.specify_host_user_runas(
         &system::Hostname::fake("host"),
         &Named("user"),
-        &Named("root"),
+        Some(&Named("root")),
     );
 
     assert!(!sudoers.settings.env_editor());
@@ -295,7 +311,7 @@ fn default_set_test() {
     sudoers.specify_host_user_runas(
         &system::Hostname::fake("host"),
         &Named("user"),
-        &Named("root"),
+        Some(&Named("root")),
     );
 
     assert_eq!(
@@ -328,7 +344,7 @@ fn default_multi_test() {
     sudoers.specify_host_user_runas(
         &system::Hostname::fake("host"),
         &Named("user"),
-        &Named("root"),
+        Some(&Named("root")),
     );
 
     assert!(!sudoers.settings.env_editor());
@@ -346,6 +362,39 @@ fn invalid_directive() {
     parse_eval::<ast::Sudo>("User_Alias, user Alias = user1, user2");
 }
 
+#[test]
+#[should_panic = "embedded $ in username"]
+fn invalid_username() {
+    parse_eval::<ast::Sudo>("User_Alias FOO = $dollar");
+}
+
+#[test]
+fn inclusive_username() {
+    let UserSpecifier::User(Identifier::Name(sirin)) = parse_eval::<ast::UserSpecifier>("şirin")
+    else {
+        panic!();
+    };
+
+    assert_eq!(sirin, "şirin");
+}
+
+#[test]
+fn sudoedit_recognized() {
+    let CommandSpec(_, Qualified::Allow(Meta::Only((cmd, args)))) =
+        parse_eval::<ast::CommandSpec>("sudoedit /etc/tmux.conf")
+    else {
+        panic!();
+    };
+    assert_eq!(cmd.as_str(), "sudoedit");
+    assert_eq!(args.unwrap().as_ref(), &["/etc/tmux.conf"][..]);
+}
+
+#[test]
+#[should_panic = "list does not take arguments"]
+fn list_does_not_take_args() {
+    parse_eval::<ast::CommandSpec>("list /etc/tmux.conf");
+}
+
 #[test]
 fn directive_test() {
     let y = parse_eval::<Spec<UserSpecifier>>;
@@ -467,6 +516,12 @@ fn specific_defaults() {
     assert!(try_parse_line("Defaults >user !use_pty").is_none());
 }
 
+#[test]
+fn at_sign_ambiguity() {
+    assert!(parse_line("Defaults@host env_keep=ALL").is_decl());
+    assert!(parse_line("defaults@host env_keep=ALL").is_spec());
+}
+
 #[test]
 fn default_specific_test() {
     let sudoers = || {
@@ -490,7 +545,7 @@ fn default_specific_test() {
     base_sudoers.specify_host_user_runas(
         &system::Hostname::fake("generic"),
         &Named("generic"),
-        &Named("generic"),
+        Some(&Named("generic")),
     );
 
     assert!(base_sudoers.settings.env_editor());
@@ -502,7 +557,7 @@ fn default_specific_test() {
     mod_sudoers.specify_host_user_runas(
         &system::Hostname::fake("host"),
         &Named("user"),
-        &Named("root"),
+        Some(&Named("root")),
     );
     assert!(!mod_sudoers.settings.env_editor());
     assert!(mod_sudoers.settings.use_pty());
@@ -513,7 +568,7 @@ fn default_specific_test() {
     mod_sudoers.specify_host_user_runas(
         &system::Hostname::fake("machine"),
         &Named("admin"),
-        &Named("runas"),
+        Some(&Named("runas")),
     );
     assert!(mod_sudoers.settings.env_editor());
     assert!(!mod_sudoers.settings.use_pty());
@@ -526,7 +581,7 @@ fn default_specific_test() {
     mod_sudoers.specify_host_user_runas(
         &system::Hostname::fake("machine"),
         &Named("admin"),
-        &Named("self"),
+        Some(&Named("self")),
     );
     mod_sudoers.specify_command(Path::new("/usr/bin/rr"), &["thrice".to_string()]);
     assert!(mod_sudoers.settings.env_editor());
